/* some strings not translated elsewhere */
static void
translate_me ()
{
	/* West; translation should be a single letter */
	_("W");
	/* North; translation should be a single letter */
	_("N");
	/* East; translation should be a single letter */
	_("E");
	/* South; translation should be a single letter */
	_("S");

	_("NT");
	/* Spades; translation should be a single letter */
	_("Sp");
	/* Hearts; translation should be a single letter */
	_("He");
	/* Diamonds; translation should be a single letter */
	_("Di");
	/* Clubs; translation should be a single letter */
	_("Cl");
}
